package struct_utils

import (
	"encoding/json"
	"fmt"
)

// UnmarshalJSON performs a JSON unmarshalling, but on type mismatches it returns a more user friendly error.
// Used exactly the same as json.Unmarshal
func UnmarshalJSON(data []byte, val interface{}) error {
	err := json.Unmarshal(data, &val)
	if err != nil {
		typeErr, ok := err.(*json.UnmarshalTypeError)
		if ok {
			return fmt.Errorf("invalid type received for field '%s': expected a value of type '%s', but received type '%s'", typeErr.Field, typeErr.Type.Name(), typeErr.Value)
		}
	}

	return err
}
